package jalview.io;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.File;

import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import jalview.datamodel.AlignmentAnnotation;
import jalview.datamodel.AlignmentI;
import jalview.datamodel.PDBEntry;
import jalview.datamodel.SequenceFeature;
import jalview.datamodel.SequenceI;
import jalview.gui.AlignFrame;
import jalview.gui.Desktop;
import jalview.structure.StructureMapping;
import jalview.structure.StructureSelectionManager;

public class AnnotatedPDBFileInputTest
{

  AlignmentI al;

  String pdbStr = "examples/1gaq.txt";

  String pdbId;

  @Before
  public void setup() throws Exception
  {
    FileLoader loader = new FileLoader(false);
    AlignFrame af = loader.LoadFileWaitTillLoaded(pdbStr,
            FormatAdapter.FILE);
    al = af.getViewport().getAlignment();
    pdbId = al.getSequenceAt(0).getDatasetSequence().getPDBId()
            .get(0).getId();
  }

  @Test
  public void checkNoDuplicates()
  {
    // not strictly a requirement, but strange things may happen if multiple
    // instances of the same annotation are placed in the alignment annotation
    // vector
    assertNotNull(al.getAlignmentAnnotation());
    // verify that all sequence annotation is doubly referenced
    AlignmentAnnotation[] avec = al.getAlignmentAnnotation();
    for (int p = 0; p < avec.length; p++)
    {
      for (int q = p + 1; q < avec.length; q++)
      {
        assertNotEquals(
                "Found a duplicate annotation row " + avec[p].label,
                avec[p], avec[q]);
      }
    }
  }

  @Test
  public void checkPDBannotationSource()
  {

    for (SequenceI asq : al.getSequences())
    {
      for (AlignmentAnnotation aa : asq.getAnnotation())
      {

        System.out.println("CalcId: " + aa.getCalcId());
        assertTrue(MCview.PDBfile.isCalcIdForFile(aa, pdbId));
      }
    }
  }

  /**
   * Check sequence features have been added
   */
  @Test
  public void checkPDBSequenceFeatures()
  {
    StructureSelectionManager ssm = StructureSelectionManager
            .getStructureSelectionManager(Desktop.instance);
    StructureMapping[] mappings = ssm.getMapping("1gaq");
    // suspect we really want to make assertions on sequence features
    // in these mappings' sequencess
    /*
     * 1GAQ/A
     */
    SequenceFeature[] sf = al.getSequenceAt(0).getSequenceFeatures();
    assertEquals(296, sf.length);
    assertEquals("RESNUM", sf[0].getType());
    assertEquals("GLU:19 1gaqA", sf[0].getDescription());
    assertEquals("RESNUM", sf[295].getType());
    assertEquals("TYR:314 1gaqA", sf[295].getDescription());

    /*
     * 1GAQ/B
     */
    sf = al.getSequenceAt(1).getSequenceFeatures();
    assertEquals(98, sf.length);
    assertEquals("RESNUM", sf[0].getType());
    assertEquals("ALA:1 1gaqB", sf[0].getDescription());
    assertEquals("RESNUM", sf[97].getType());
    assertEquals("ALA:98 1gaqB", sf[97].getDescription());

    /*
     * 1GAQ/C
     */
    sf = al.getSequenceAt(2).getSequenceFeatures();
    assertEquals(296, sf.length);
    assertEquals("RESNUM", sf[0].getType());
    assertEquals("GLU:19 1gaqC", sf[0].getDescription());
    assertEquals("RESNUM", sf[295].getType());
    assertEquals("TYR:314 1gaqC", sf[295].getDescription());
  }

  @Test
  public void checkAnnotationWiring()
  {
    assertTrue(al.getAlignmentAnnotation() != null);
    // verify that all sequence annotation is doubly referenced
    for (AlignmentAnnotation aa : al.getAlignmentAnnotation())
    {
      if (aa.sequenceRef != null)
      {
        assertTrue(al.getSequences().contains(aa.sequenceRef));
        assertNotNull(aa.sequenceRef.getAnnotation());
        boolean found = false;
        for (AlignmentAnnotation sqan : aa.sequenceRef.getAnnotation())
        {
          if (sqan == aa)
          {
            found = true;
            break;
          }
        }
        assertTrue(
                "Couldn't find sequence associated annotation "
                        + aa.label
                        + " on the sequence it is associated with.\nSequence associated editing will fail.",
                found);
      }
    }
  }

  /**
   * @throws java.lang.Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {
    jalview.bin.Jalview.main(new String[]
    { "-props", "test/src/jalview/io/testProps.jvprops" });
  }

  /**
   * @throws java.lang.Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
    jalview.gui.Desktop.instance.closeAll_actionPerformed(null);

  }

  @Test
  public void testJalviewProjectRelocationAnnotation() throws Exception
  {

    String inFile = "examples/1gaq.txt";
    String tfile = File.createTempFile("JalviewTest", ".jvp")
            .getAbsolutePath();
    AlignFrame af = new jalview.io.FileLoader().LoadFileWaitTillLoaded(
            inFile, FormatAdapter.FILE);
    assertTrue("Didn't read input file " + inFile, af != null);
    assertTrue("Failed to store as a project.",
            af.saveAlignment(tfile, "Jalview"));
    af.closeMenuItem_actionPerformed(true);
    af = null;
    af = new jalview.io.FileLoader().LoadFileWaitTillLoaded(tfile,
            FormatAdapter.FILE);
    assertTrue("Failed to import new project", af != null);
    for (SequenceI asq : af.getViewport().getAlignment().getSequences())
    {
      SequenceI sq = asq;
      while (sq.getDatasetSequence() != null)
      {
        sq = sq.getDatasetSequence();
      }
      assertNotNull(sq.getPDBId());
      assertEquals("Expected only one PDB ID", sq.getPDBId().size(), 1);
      for (PDBEntry pdbentry : sq.getPDBId())
      {
        System.err.println("PDB Entry " + pdbentry.getId() + " "
                + pdbentry.getFile());
        boolean exists = false, found = false;
        for (AlignmentAnnotation ana : sq.getAnnotation())
        {
          System.err.println("CalcId " + ana.getCalcId());
          if (ana.getCalcId() != null
                  && MCview.PDBfile.isCalcIdHandled(ana.getCalcId()))
          {
            exists = true;
            if (MCview.PDBfile.isCalcIdForFile(ana,
                    pdbentry.getId()))
            {
              found = true;
            }
          }
        }
        if (exists)
        {
          assertTrue("Couldn't find any annotation for " + pdbentry.getId()
                  + " (file handle " + pdbentry.getFile() + ")", found);
        }
      }
    }
  }
}
